current = "1.0"
GFFv = {'1.0': ["iso_5p", "iso_3p", "iso_add", "iso_snp_seed",
                "iso_snp_central_offset", "iso_snp_central",
                "iso_snp_central_supp", "iso_snp", "NA"]}
ATTRv = {'1.0': set(('IDU', 'Name', 'Alias', 'Parent',
                     'Read', 'Variant', 'Changes',
                     'Cigar', 'Hits', 'Genomic',
                     'Expression', 'Filter', 'Cigar',
                     'Seed_fam'))}
